==> etherx-0.7.tar.gz: etherx-0.7/daemon/etherxd.c <==
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "etherxd.h"
#include <getopt.h>


/* code for an etherx daemon
    - loads config file
    - error and connection log file
    - DSO loads transports
    - listens on sockets and manages connections
    - routes epackets between connections and transports
*/

int main(int argc, char *argv[])
{
    pth_event_t sigevt;
    sigset_t set;
    int sig, help;
    char *log, *registry = NULL;
    struct option options[] =
    {
        {"registry",1,0,'r'},
        {0,0,0,0}
    };

    help = opterr = 0;
    while((sig = getopt_long(argc,argv,"r:D",options,NULL)) > 0)
    {
        switch(sig)
        {
            case 'r':
                registry = strdup(optarg);
                break;
            case 'D':
                jlib_debug();
                break;
            default:
                help = 1;
        }
    }

    if(help)
    {
        printf("Usage:\netherxd [-r registry.xml] [-D]\n");
        exit(0);
    }

    if(registry == NULL)
        registry = strdup(SYSCONFDIR "/etherx.xml");

    registry_load(registry);

    log = xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(xmlnode_get_tag(config,"etherx"),"error")));
    if(log == NULL)
        jlib_init(JLIB_LOG_STDERR, NULL);
    else if(j_strcmp(log,"syslog") == 0)
        jlib_init(JLIB_LOG_SYSLOG, "etherxd");
    else
        jlib_init(JLIB_LOG_FILE, log);

    logger_init();
    if(net_init() <= 0)
    {
        printf("Unable to bind to port 5269.  Either check your <listen></listen> directive(s) for validity or verify that an existing etherxd isn't running (use 'netstat -ln' and look for ':5269')\n");
        exit(0);
    }
    registry_scan();

    sigemptyset(&set);
    sigaddset(&set, SIGHUP);
    pth_sigmask(SIG_UNBLOCK, &set, NULL);
    sigevt = pth_event(PTH_EVENT_SIGS, &set, &sig);

    /* main thread, wait on restart/kill signals and react appropriately */
    jwarn("etherxd","server started");
    while (1)
    {
        pth_wait(sigevt);
        jwarn("etherxd","SIGHUP: reloading registry and resetting transports");
        registry_load(registry);
        registry_scan();
    }
    return 0;
}

==> etherx-0.8.tar.gz: etherx-0.8/daemon/etherxd.c <==
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "etherxd.h"
#include <getopt.h>


/* code for an etherx daemon
    - loads config file
    - error and connection log file
    - DSO loads transports
    - listens on sockets and manages connections
    - routes epackets between connections and transports
*/

int main(int argc, char *argv[])
{
    pth_event_t sigevt;
    sigset_t set;
    int sig, help;
    char *log, *cfgfile = NULL;
    struct option options[] =
    {
        {"config",1,0,'c'},
        {0,0,0,0}
    };

    help = opterr = 0;
    while((sig = getopt_long(argc,argv,"c:D",options,NULL)) > 0)
    {
        switch(sig)
        {
            case 'c':
                cfgfile = strdup(optarg);
                break;
            case 'D':
                jlib_debug();
                break;
            default:
                help = 1;
        }
    }

    if(help)
    {
        printf("Usage:\netherxd [-c etherx.xml] [-D]\n");
        exit(0);
    }

    if(cfgfile == NULL)
        cfgfile = strdup(SYSCONFDIR "/etherx.xml");

    config_load(cfgfile);

    log = xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(xmlnode_get_tag(config,"etherx"),"error")));
    if(log == NULL)
        jlib_init(JLIB_LOG_STDERR, NULL);
    else if(j_strcmp(log,"syslog") == 0)
        jlib_init(JLIB_LOG_SYSLOG, "etherxd");
    else
        jlib_init(JLIB_LOG_FILE, log);

    logger_init();
    if(manager_init() <= 0)
    {
        printf("Unable to bind to port 5269.  Either check your <listen></listen> directive(s) for validity or verify that an existing etherxd isn't running (use 'netstat -ln' and look for ':5269')\n");
        exit(0);
    }
    dso_init();

    sigemptyset(&set);
    sigaddset(&set, SIGHUP);
    sigaddset(&set, SIGINT);
    sigaddset(&set, SIGTERM);
    pth_sigmask(SIG_UNBLOCK, &set, NULL);
    sigevt = pth_event(PTH_EVENT_SIGS, &set, &sig);

    /* main thread, wait on kill signals and react appropriately */
    jwarn("etherxd","server started");
    pth_wait(sigevt);
    jwarn("etherxd","received kill signal, shutting down");
    /* XXX need to shut down gracefully, empty buffers, etc */
    pth_yield(NULL); /* one last chance to make sure everyone is done */
    pth_kill();
    return 0;
}
